// Copyright © 2021 Finn Herzfeld <finn@janky.solutions>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

package link

import (
	"encoding/json"
	"log"
	"os"

	"github.com/jedib0t/go-pretty/v6/table"
	"github.com/mdp/qrterminal"
	"github.com/spf13/cobra"
	"gopkg.in/yaml.v2"

	"gitlab.com/signald/signald-go/cmd/signaldctl/common"
	"gitlab.com/signald/signald-go/signald"
	"gitlab.com/signald/signald-go/signald/client-protocol/v0"
)

var (
	noWait bool

	LinkAccountCmd = &cobra.Command{
		Use:   "link",
		Short: "create a local account by linking to an existing Signal account",
		Run: func(_ *cobra.Command, _ []string) {
			requestID := signald.GenerateID()
			err := common.Signald.RawRequest(v0.LegacyRequest{Type: "link", ID: requestID})
			if err != nil {
				log.Fatal("error sending request: ", err)
			}
			c := make(chan v0.LegacyResponse)
			go common.Signald.Listen(c)
			uri := signald.GetLegacyResponse(c, requestID)
			if uri.Type != "linking_uri" {
				log.Fatalf("unexpected response from signald when requesting link: %+v", uri)
			}

			switch common.OutputFormat {
			case common.OutputFormatJSON:
				err := json.NewEncoder(os.Stdout).Encode(uri.Data.URI)
				if err != nil {
					log.Fatal(err, "error encoding response to stdout")
				}
			case common.OutputFormatYAML:
				err := yaml.NewEncoder(os.Stdout).Encode(uri.Data.URI)
				if err != nil {
					log.Fatal(err, "error encoding response to stdout")
				}
			case common.OutputFormatCSV, common.OutputFormatTable:
				t := table.NewWriter()
				t.SetOutputMirror(os.Stdout)
				t.AppendHeader(table.Row{"URI"})
				t.AppendRow(table.Row{uri.Data.URI})
				if common.OutputFormat == common.OutputFormatCSV {
					t.RenderCSV()
				} else {
					t.SetStyle(table.StyleLight)
					t.Render()
				}
			case common.OutputFormatQR, common.OutputFormatDefault:
				qrterminal.Generate(uri.Data.URI, qrterminal.M, os.Stdout)
			default:
				log.Fatal("unsupported output format")
			}

			if noWait {
				return
			}

			finish := signald.GetLegacyResponse(c, requestID)
			if finish.Type == "linking_successful" {
				log.Println("linking successful")
				return
			}
			if finish.Type == "linking_error" {
				log.Fatal("error from signald:", finish.Data.Message)
			} else {
				log.Fatal("unexpected message from signald:", finish.Data.Message)
			}
		},
	}
)

func init() {
	LinkAccountCmd.Flags().BoolVar(&noWait, "no-wait", false, "return after the linking URI is printed to stdout. By default, signaldctl will wait until the server acknowledges that the linking has completed before exiting")
}
