// Copyright © 2021 Finn Herzfeld <finn@janky.solutions>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

package list

import (
	"encoding/json"
	"log"
	"os"

	"github.com/jedib0t/go-pretty/v6/table"
	"github.com/spf13/cobra"
	"gopkg.in/yaml.v2"

	"gitlab.com/signald/signald-go/cmd/signaldctl/common"
	"gitlab.com/signald/signald-go/signald"
	"gitlab.com/signald/signald-go/signald/client-protocol/v0"
)

var (
	ListAccountCmd = &cobra.Command{
		Use:   "list",
		Short: "return a list of accounts",
		Run: func(_ *cobra.Command, _ []string) {
			requestID := signald.GenerateID()
			err := common.Signald.RawRequest(v0.LegacyRequest{
				Type: "list_accounts",
				ID:   requestID,
			})
			if err != nil {
				log.Fatal("error sending request: ", err)
			}

			c := make(chan v0.LegacyResponse)
			go common.Signald.Listen(c)

			var accounts []v0.Account
			for {
				message := <-c
				if message.ID == requestID {
					accounts = message.Data.Accounts
					break
				}
			}

			switch common.OutputFormat {
			case common.OutputFormatJSON:
				err := json.NewEncoder(os.Stdout).Encode(accounts)
				if err != nil {
					log.Fatal(err, "error encoding response to stdout")
				}
			case common.OutputFormatYAML:
				err := yaml.NewEncoder(os.Stdout).Encode(accounts)
				if err != nil {
					log.Fatal(err, "error encoding response to stdout")
				}
			case common.OutputFormatCSV, common.OutputFormatTable, common.OutputFormatDefault:
				t := table.NewWriter()
				t.SetOutputMirror(os.Stdout)
				t.AppendHeader(table.Row{"Phone Number", "UUID", "Device ID", "Subscribed"})

				for _, account := range accounts {
					t.AppendRow(table.Row{account.Username, account.UUID, account.DeviceID, account.Subscribed})
				}

				if common.OutputFormat == common.OutputFormatCSV {
					t.RenderCSV()
				} else {
					t.SetStyle(table.StyleLight)
					t.Render()
				}
			default:
				log.Fatal("Unsupported output format")
			}
		},
	}
)
